package cbg.editor.test;
import junit.framework.TestCase;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.widgets.Shell;

import cbg.editor.ColoringEditorTools;
import cbg.editor.ColoringPartitionScanner;
import cbg.editor.ColoringSourceViewerConfiguration;
import cbg.editor.prefs.ColorsPreferencePage;
import cbg.editor.rules.ColorManager;
public class DocumentPartitionerTest extends TestCase {
	public void testDocumentPartitioner() {
		RuleBasedScanner scanner = config.getCodeScanner();
		assertNotNull(scanner);
		String code = "public static void main(";
		scanner.setRange(new Document(code), 0, code.length());
		checkPublicKeyword(scanner);
		checkWhitespaceWord(scanner);
		checkStaticKeyword(scanner);
		checkWhitespaceWord(scanner);
		checkVoidKeyword(scanner);
		checkWhitespaceWord(scanner);
	}

	public void testCase() {
		RuleBasedScanner scanner = config.getCodeScanner();
		assertNotNull(scanner);
		String code = "implements";
		scanner.setRange(new Document(code), 0, code.length());
		IToken keywordToken = scanner.nextToken();
		TextAttribute keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.KEYWORD1_COLOR), keywordAttribute.getForeground());
		assertEquals("implements".length(), scanner.getTokenLength());
		assertEquals(0, scanner.getTokenOffset());
		
		code = "Implements";
		scanner.setRange(new Document(code), 0, code.length());
		keywordToken = scanner.nextToken();
		keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.NULL_COLOR), keywordAttribute.getForeground());
		assertEquals("implements".length(), scanner.getTokenLength());
		assertEquals(0, scanner.getTokenOffset());
	}
	private void checkMain(RuleBasedScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		TextAttribute keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.FUNCTION_COLOR), keywordAttribute.getForeground());
		assertEquals("main".length(), scanner.getTokenLength());
		assertEquals(19, scanner.getTokenOffset());
	}
	private void checkVoidKeyword(RuleBasedScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		TextAttribute keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.KEYWORD3_COLOR), keywordAttribute.getForeground());
		assertEquals("void".length(), scanner.getTokenLength());
		assertEquals(14, scanner.getTokenOffset());
	}
	private void checkStaticKeyword(RuleBasedScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		TextAttribute keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.KEYWORD1_COLOR), keywordAttribute.getForeground());
		assertEquals("static".length(), scanner.getTokenLength());
		assertEquals(7, scanner.getTokenOffset());
	}
	private void checkWhitespaceWord(RuleBasedScanner scanner) {
		IToken whiteToken = scanner.nextToken();
		assertTrue(whiteToken.isWhitespace());
		assertEquals(" ".length(), scanner.getTokenLength());
	}
	private void checkPublicKeyword(RuleBasedScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		TextAttribute keywordAttribute = (TextAttribute) keywordToken.getData();
		assertEquals(colorManager.getColor(ColorsPreferencePage.KEYWORD1_COLOR), keywordAttribute.getForeground());
		assertEquals("public".length(), scanner.getTokenLength());
		assertEquals(0, scanner.getTokenOffset());
	}
	private ColorManager colorManager;
	private ColoringEditorTools tools;
	private ColoringSourceViewerConfiguration config;
	private IPresentationReconciler reconciler;
	private IDocumentPartitioner partitioner;
	public DocumentPartitionerTest(String name) {
		super(name);
	}
	public static void main(String[] args) {
		junit.awtui.TestRunner.run(DocumentPartitionerTest.class);
	}
	private void checkClassDef(ColoringPartitionScanner scanner) {
	}
	private String getRegionSubstring(String code, ITypedRegion region) {
		return code.substring(region.getOffset(), region.getOffset() + region.getLength());
	}
	private ITypedRegion[] partitionCode(String code) {
		IDocument document = new Document(code);
		partitioner.connect(document);
		ITypedRegion[] regions = partitioner.computePartitioning(0, code.length());
		return regions;
	}
	protected void setUp() throws Exception {
		super.setUp();
		IPreferenceStore store = new PreferenceStore();
		ColorManager.initDefaultColors(store);
		colorManager = new ColorManager(store);
		tools = new ColoringEditorTools();
		config = new ColoringSourceViewerConfiguration(colorManager, tools);
		config.setFilename("foo.java");
		ISourceViewer viewer = new SourceViewer(new Shell(), null, 0);
		reconciler = config.getPresentationReconciler(viewer);
	}
	protected void tearDown() throws Exception {
		super.tearDown();
	}
}
