package cbg.editor.test;

import junit.framework.TestCase;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.widgets.Shell;
import cbg.editor.ColoringEditorTools;
import cbg.editor.ColoringPartitionScanner;
import cbg.editor.ColoringPartitioner;
import cbg.editor.ColoringSourceViewerConfiguration;
import cbg.editor.Modes;
import cbg.editor.jedit.Type;
import cbg.editor.rules.ColorManager;

public class MarkFollowingTest extends TestCase {
	private ColorManager colorManager;
	private ColoringEditorTools tools;
	private ColoringSourceViewerConfiguration config;
	private IPresentationReconciler reconciler;
	private ColoringPartitionScanner scanner;
	private IDocumentPartitioner partitioner;
	public MarkFollowingTest(String name) {
		super(name);
	}
	public static void main(String[] args) {
		junit.awtui.TestRunner.run(MarkFollowingTest.class);
	}
	public void testMarkFollowing() {
        assertNotNull(scanner);
        String code = "#topbar {}";
        scanner.setRange(new Document(code), 0, code.length());

        checkMarkFollowing(scanner);
	}

	public void testMarkFollowingFalse() {
	    assertNotNull(scanner);
	    String code = "topbar {}";
	    scanner.setRange(new Document(code), 0, code.length());
	
	    checkMarkFollowingFalse(scanner);
	}

	private void checkMarkFollowingFalse(ColoringPartitionScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		assertTrue(keywordToken.isOther());
		assertEquals(1, scanner.getTokenLength());
		assertEquals(0, scanner.getTokenOffset());
	}

	private void checkMarkFollowing(ColoringPartitionScanner scanner) {
		IToken keywordToken = scanner.nextToken();
		String data = (String) keywordToken.getData();
		assertEquals(Type.MARK_FOLLOWING + "@1" + "." + ColoringPartitionScanner.LITERAL2, data);
		assertEquals("#topbar".length(), scanner.getTokenLength());
		assertEquals(0, scanner.getTokenOffset());
	}
    
    private String getRegionSubstring(String code, ITypedRegion region) {
        return code.substring(region.getOffset(), region.getOffset() + region.getLength());
    }

	private ITypedRegion[] partitionCode(String code) {
		IDocument document = new Document(code);
        partitioner.connect(document);
		ITypedRegion[] regions = partitioner.computePartitioning(0, code.length());
        return regions;
	}
	
	protected void setUp() throws Exception {
		super.setUp();
		IPreferenceStore store = new PreferenceStore();
        ColorManager.initDefaultColors(store);
        colorManager = new ColorManager(store);
        tools = new ColoringEditorTools();
        config = new ColoringSourceViewerConfiguration(colorManager, tools);
        config.setFilename("foo.css");
        scanner = new ColoringPartitionScanner(Modes.getMode("css.xml"));
		partitioner = new ColoringPartitioner(scanner, scanner.getContentTypes());
        ISourceViewer viewer = new SourceViewer(new Shell(), null, 0);
        reconciler = config.getPresentationReconciler(viewer);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

}
